unit uCiaLogon;

interface

uses
   SysUtils, Classes;

type
  TOnRcvdMatch = procedure(Sender: TObject; Event: word) of object;
  ECiaLogon = class(Exception);
  TCiaLogon = class(TComponent)
  private
    FBuf: PChar;
    FBufStart: integer;
    FBufLen: integer;
    FEvents: TStringList;
    FIgnoreCase: boolean;
    FOnRcvdMatch: TOnRcvdMatch;
    procedure SetBufLen(Value: integer);
    function  LookUp(const ToSeek: PChar): boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure Add(Data: PChar; Len: integer);
    function  AddEvent(const EventStr: string): word;
    procedure Clear;
    procedure ClearEvents;
    procedure RemoveEvent(Index: word);
  published
    property Len: integer read FBufLen write SetBufLen;
    property IgnoreCase: boolean read FIgnoreCase write FIgnoreCase;
    property OnRcvdMatch: TOnRcvdMatch read FOnRcvdMatch write FOnRcvdMatch;
  end;

procedure Register;

implementation

//------------------------------------------------------------------------------
procedure Register;
begin
   RegisterComponents('Cia', [TCiaLogon]);
end;

//------------------------------------------------------------------------------
constructor TCiaLogon.Create(AOwner: TComponent);
begin
   inherited;
   SetBufLen(80);
   FEvents := TStringList.Create;
end;

//------------------------------------------------------------------------------
destructor TCiaLogon.Destroy;
begin
   if Assigned(FBuf) then
      FreeMem(FBuf);
   if Assigned(FEvents) then
      FEvents.Destroy;
   inherited;
end;

//------------------------------------------------------------------------------
procedure TCiaLogon.Add(Data: PChar; Len: integer);
var
   n, Rest: integer;
begin
   if Len > FBufLen then
      raise ECiaLogon.Create('Data to large for buffer');

   Rest := FBufLen - FBufStart;
   if Rest >= Len then
   begin
      Move(Data[0], FBuf[FBufStart], Len);
      Inc(FBufStart, Len);
      if FBufStart >= FBufLen then
         FBufStart := 0;
   end else
   begin
      Move(Data[0], FBuf[FBufStart], Rest);
      FBufStart := Len - Rest;
      Move(Data[Rest], FBuf[0], FBufStart);
   end;

   for n := 0 to FEvents.Count - 1 do
      if LookUp(PChar(FEvents[n])) and Assigned(FOnRcvdMatch) then
      begin
         FOnRcvdMatch(Self, n);
         FEvents[n] := '';
         Exit;
      end;

end;

//------------------------------------------------------------------------------
function CmpCharCI(a, b: char): boolean;
begin
   if a in ['A'..'Z'] then
   asm
      mov al, a
      or  al, 20h
      mov a, al
   end;

   if b in ['A'..'Z'] then
   asm
      mov al, b
      or  al, 20h
      mov b, al
   end;

   result := a = b;
end;

//------------------------------------------------------------------------------
function TCiaLogon.LookUp(const ToSeek: PChar): boolean;
var
   i, j, k: integer;
   Len: integer;
   Same: boolean;
begin
   Len := Length(ToSeek);
   if Len = 0 then
   begin
      Result := False;
      Exit;
   end;

   Result := True;
   i := FBufStart;
   repeat
      if FIgnoreCase then
         Same := CmpCharCI(ToSeek[0], FBuf[i])
      else
         Same := ToSeek[0] = FBuf[i];
      if Same then
      begin
         // first char match, check if the rest match also
         j := 1;
         k := i + 1;
         while j < Len do
         begin
            if k >= FBufLen then
               k := 0;
            if FIgnoreCase then
               Same := CmpCharCI(ToSeek[j], FBuf[k])
            else
               Same := ToSeek[j] = FBuf[k];
            if not Same then
               break;
            Inc(j);
            Inc(k);
         end;
         if j >= Len then
            Exit;    // match
      end;
      Inc(i);
      if i >= FBufLen then
         i := 0;
   until i = FBufStart;

   Result := False;
end;

//------------------------------------------------------------------------------
procedure TCiaLogon.Clear;
begin
   FBufStart := 0;
   FillChar(FBuf^, FBufLen, 0);
end;

//------------------------------------------------------------------------------
procedure TCiaLogon.SetBufLen(Value: integer);
begin
   // Should be the size of the largest chucnk you will receive one at the time
   // But also as small as possible to not looze performance
   if Value = FBufLen then
      Exit;
   ReAllocMem(FBuf, Value);
   FBufLen := Value;
   Clear;
end;

//------------------------------------------------------------------------------
function TCiaLogon.AddEvent(const EventStr: string): word;
begin
   Result := FEvents.Add(EventStr);
end;

//------------------------------------------------------------------------------
procedure TCiaLogon.RemoveEvent(Index: word);
begin
   FEvents[Index] := '';
end;

//------------------------------------------------------------------------------
procedure TCiaLogon.ClearEvents;
begin
   FEvents.Clear;
end;

end.
